"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const injector_1 = require("../injector");
const logger_1 = require("../../services/logger");
const injectable_decorator_1 = require("../../decorators/services/injectable.decorator");
describe('Injector', () => {
    let logger;
    let instance;
    before(() => {
        logger = new logger_1.Logger('abc-xyz', 'default', false);
        instance = new injector_1.Injector(logger);
    });
    describe('.resolveInjectable', () => {
        it('should provide the logger when requested', () => {
            chai_1.expect(instance.resolveInjectable(logger_1.Logger)).to.equal(logger);
        });
        it('should provide itself when requested', () => {
            chai_1.expect(instance.resolveInjectable(injector_1.Injector)).to.equal(instance);
        });
        it('should instantiate a class that has not been created before', () => {
            let ctorCalled = false;
            let TestClass = class TestClass {
                constructor() { ctorCalled = true; }
            };
            TestClass = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [])
            ], TestClass);
            chai_1.expect(instance.resolveInjectable(TestClass)).to.be.an.instanceOf(TestClass);
            chai_1.expect(ctorCalled).to.be.true;
        });
        it('should not instantiate a class more than once', () => {
            let ctorCallCount = 0;
            let TestClass = class TestClass {
                constructor() { ctorCallCount++; }
            };
            TestClass = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [])
            ], TestClass);
            let testInst = instance.resolveInjectable(TestClass);
            chai_1.expect(testInst).to.be.an.instanceOf(TestClass);
            chai_1.expect(ctorCallCount).to.eq(1);
            chai_1.expect(instance.resolveInjectable(TestClass)).to.eq(testInst);
            chai_1.expect(ctorCallCount).to.eq(1);
        });
        it('should return undefined if you try to inject a falsey type', () => {
            chai_1.expect(instance.resolveInjectable(null)).to.be.undefined;
        });
        it('should inject constructor parameters when it instantiates a type', () => {
            let TestClassInner = class TestClassInner {
                constructor() { }
            };
            TestClassInner = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [])
            ], TestClassInner);
            let TestClassOuter = class TestClassOuter {
                constructor(inner) {
                    this.inner = inner;
                }
            };
            TestClassOuter = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [TestClassInner])
            ], TestClassOuter);
            let outer = instance.resolveInjectable(TestClassOuter);
            chai_1.expect(outer).not.to.be.undefined;
            chai_1.expect(outer).to.be.an.instanceOf(TestClassOuter);
            chai_1.expect(outer.inner).to.be.an.instanceOf(TestClassInner);
        });
        it('should throw when the constructor parameters cannot be resolved', () => {
            let TestClass = class TestClass {
                constructor(dependency) {
                    this.dependency = dependency;
                }
            };
            TestClass = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [Object])
            ], TestClass);
            chai_1.expect(() => instance.resolveInjectable(TestClass)).to.throw(/Failed to resolve constructor types/);
        });
    });
    describe('.provide', () => {
        it('should throw when the provide value is falsey', () => {
            chai_1.expect(() => instance.provide(null)).to.throw(/Invalid ProvideMetadata:/);
        });
        it('should throw when the provide constructor function is falsey', () => {
            chai_1.expect(() => instance.provide({ provide: null, useValue: null })).to.throw(/provide a value for a falsey type/);
        });
        it('should throw when multiple values for a single type are provided', () => {
            let TestClass = class TestClass {
                constructor() { }
            };
            TestClass = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [])
            ], TestClass);
            instance.provide({ provide: TestClass, useValue: null });
            chai_1.expect(() => instance.provide({ provide: TestClass, useValue: null })).to.throw(/Duplicate value provided/);
        });
        it('should dependency inject a replacement class if useClass option present', () => {
            let TestClass = class TestClass {
                constructor() { }
            };
            TestClass = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [])
            ], TestClass);
            let FauxTestClass = class FauxTestClass {
                constructor() { }
            };
            FauxTestClass = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [])
            ], FauxTestClass);
            ;
            instance.provide({ provide: TestClass, useClass: FauxTestClass });
            chai_1.expect(instance.resolveInjectable(TestClass)).to.be.an.instanceOf(FauxTestClass);
        });
        it('should dependency inject a replacement value if useValue option present', () => {
            let TestClass = class TestClass {
                constructor() { }
            };
            TestClass = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [])
            ], TestClass);
            let replacementValue = { myObj: 'someVal' };
            instance.provide({ provide: TestClass, useValue: replacementValue });
            chai_1.expect(instance.resolveInjectable(TestClass)).to.eq(replacementValue);
        });
        it('should dependency inject a falsey value if useValue option is not undefined', () => {
            let TestClass = class TestClass {
                constructor() { }
            };
            TestClass = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [])
            ], TestClass);
            instance.provide({ provide: TestClass, useValue: null });
            chai_1.expect(instance.resolveInjectable(TestClass)).to.be.null;
        });
        it('should dependency inject a factory function if useCallback is present', () => {
            let TestClass = class TestClass {
                constructor() { }
            };
            TestClass = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [])
            ], TestClass);
            let testInst = new TestClass();
            instance.provide({ provide: TestClass, useCallback: () => testInst });
            chai_1.expect(instance.resolveInjectable(TestClass)).to.eq(testInst);
        });
        it('should call the factory function every time the class is resolved', () => {
            let TestClass = class TestClass {
                constructor() { }
            };
            TestClass = __decorate([
                injectable_decorator_1.Injectable(),
                __metadata("design:paramtypes", [])
            ], TestClass);
            let callbackCallCount = 0;
            instance.provide({ provide: TestClass, useCallback: () => callbackCallCount++ });
            chai_1.expect(callbackCallCount).to.eq(0);
            instance.resolveInjectable(TestClass);
            chai_1.expect(callbackCallCount).to.eq(1);
            instance.resolveInjectable(TestClass);
            chai_1.expect(callbackCallCount).to.eq(2);
        });
    });
});
//# sourceMappingURL=injector.spec.js.map